@@ -1,3 +1,4 @@
+from keyword import iskeyword
 import sphinxify
 import typing
 import yaml
@@ -6,6 +7,7 @@ from .hooks_datacfg import (
     HooksDataYaml,
     BufferType,
     ClassData,
+    EnumValue,
     FunctionData,
     PropData,
     PropAccess,
@@ -39,6 +41,15 @@ _rvp_map = {
     ReturnValuePolicy.AUTOMATIC_REFERENCE: ", py::return_value_policy::automatic_reference",
 }
 
+# fmt: off
+_operators = {
+    # binary
+    "-", "+", "*", "/", "%", "&", "^", "==", "!=", "|", ">", ">=", "<", "<=",
+    # inplace
+    "+=", "-=", "*=", "/=", "%=", "&=", "^=", "|=",
+}
+# fmt: on
+
 
 class HookError(Exception):
     pass
@@ -56,15 +67,19 @@ class Hooks:
     _qualname_bad = ":<>="
     _qualname_trans = str.maketrans(_qualname_bad, "_" * len(_qualname_bad))
 
-    def __init__(self, data: HooksDataYaml, casters: typing.Dict[str, str]):
+    def __init__(
+        self, data: HooksDataYaml, casters: typing.Dict[str, str], report_only: bool
+    ):
         self.gendata = GeneratorData(data)
         self.rawdata = data
         self.casters = casters
+        self.report_only = report_only
+        self.has_operators = False
 
-        self.types = set()
-        self.class_hierarchy = {}
+        self.types: typing.Set[str] = set()
+        self.class_hierarchy: typing.Dict[str, typing.List[str]] = {}
 
-        self.subpackages = {}
+        self.subpackages: typing.Dict[str, str] = {}
 
     def report_missing(self, name: str, reporter: MissingReporter):
         self.gendata.report_missing(name, reporter)
@@ -93,19 +108,46 @@ class Hooks:
                 includes.add(header)
         return sorted(includes)
 
-    def _set_name(self, name, data):
+    def _set_name(self, name, data, strip_prefixes=None):
         if data.rename:
             return data.rename
 
-        sp = self.rawdata.strip_prefixes
-        if sp:
-            for pfx in sp:
+        if strip_prefixes is None:
+            strip_prefixes = self.rawdata.strip_prefixes
+
+        if strip_prefixes:
+            for pfx in strip_prefixes:
                 if name.startswith(pfx):
-                    name = name[len(pfx) :]
-                    break
+                    n = name[len(pfx) :]
+                    if not n.isnumeric():
+                        name = n
+                        break
+
+        if iskeyword(name):
+            return f"{name}_"
+        elif name.isnumeric():
+            raise ValueError(f"name cannot be all numbers ({name})")
 
         return name
 
+    def _process_doc(self, thing, data) -> typing.Optional[typing.List[str]]:
+        doc = ""
+        doc_quoted: typing.Optional[typing.List[str]] = None
+
+        if data.doc is not None:
+            doc = data.doc
+        elif "doxygen" in thing:
+            doc = thing["doxygen"]
+            doc = sphinxify.process_raw(doc)
+
+        if doc:
+            # TODO
+            doc = doc.replace("\\", "\\\\").replace('"', '\\"')
+            doc_quoted = doc.splitlines(keepends=True)
+            doc_quoted = ['"%s"' % (dq.replace("\n", "\\n"),) for dq in doc_quoted]
+
+        return doc_quoted
+
     def _resolve_default(self, fn, name):
         if isinstance(name, (int, float)):
             return str(name)
@@ -144,13 +186,21 @@ class Hooks:
 
             en["x_name"] = self._set_name(ename, enum_data)
 
+        en["x_doc_quoted"] = self._process_doc(en, enum_data)
+
+        if value_prefix:
+            strip_prefixes = [value_prefix + "_", value_prefix]
+        else:
+            strip_prefixes = []
+
         for v in en["values"]:
             name = v["name"]
-            if value_prefix and name.startswith(value_prefix):
-                name = name[len(value_prefix) :]
-                if name[0] == "_":
-                    name = name[1:]
-            v["x_name"] = name
+            v_data = enum_data.values.get(name)
+            if v_data is None:
+                v_data = EnumValue()
+            v["x_name"] = self._set_name(name, v_data, strip_prefixes)
+            v["data"] = v_data
+            v["x_doc_quoted"] = self._process_doc(v, v_data)
 
     def header_hook(self, header, data):
         """Called for each header"""
@@ -173,6 +223,18 @@ class Hooks:
         data["type_caster_includes"] = self._get_type_caster_includes()
         data["class_hierarchy"] = self.class_hierarchy
         data["subpackages"] = self.subpackages
+        data["x_has_operators"] = self.has_operators
+
+        templates = {}
+        for k, tmpl_data in data["data"].templates.items():
+            qualname = tmpl_data.qualname
+            if "::" not in qualname:
+                qualname = f"::{qualname}"
+            tmpl_data = tmpl_data.dict()
+            tmpl_data["x_qualname_"] = qualname.translate(self._qualname_trans)
+            templates[k] = tmpl_data
+
+        data["templates"] = templates
 
     def _function_hook(self, fn, data: FunctionData, internal: bool = False):
         """shared with methods/functions"""
@@ -196,8 +258,8 @@ class Hooks:
         x_keepalives = []
 
         x_genlambda = False
-        x_lambda_pre = []
-        x_lambda_post = []
+        x_lambda_pre: typing.List[str] = []
+        x_lambda_post: typing.List[str] = []
 
         # Use this if one of the parameter types don't quite match
         param_override = data.param_override
@@ -249,10 +311,10 @@ class Hooks:
 
             ptype = "in"
 
-            bufinfo = buffer_params.pop(p["name"], None)
             buflen = buflen_params.pop(p["name"], None)
 
-            if bufinfo:
+            if p["name"] in buffer_params:
+                bufinfo = buffer_params.pop(p["name"])
                 x_genlambda = True
                 bname = f"__{bufinfo.src}"
                 p["constant"] = 1
@@ -305,6 +367,8 @@ class Hooks:
                 if asz:
                     p["x_type"] = "std::array<%s, %s>" % (p["x_type"], asz)
                     p["x_callname"] = "%(x_callname)s.data()" % p
+                    if "default" not in p:
+                        p["default"] = "{}"
                 else:
                     # it's a vector
                     pass
@@ -362,7 +426,8 @@ class Hooks:
         # Temporary values to store out parameters in
         if x_temps:
             for out in reversed(x_temps):
-                x_lambda_pre.insert(0, "%(x_type)s %(name)s = 0" % out)
+                odef = out.get("default", "0")
+                x_lambda_pre.insert(0, f"{out['x_type']} {out['name']} = {odef}")
 
         # Rename functions
         if data.rename:
@@ -372,24 +437,28 @@ class Hooks:
         elif fn["constructor"]:
             x_name = "__init__"
 
-        doc = ""
-        doc_quoted = ""
-
-        if data.doc is not None:
-            doc = data.doc
-        elif "doxygen" in fn:
-            doc = fn["doxygen"]
-            doc = sphinxify.process_raw(doc)
-
-        if doc:
-            # TODO
-            doc = doc.replace("\\", "\\\\").replace('"', '\\"')
-            doc_quoted = doc.splitlines(keepends=True)
-            doc_quoted = ['"%s"' % (dq.replace("\n", "\\n"),) for dq in doc_quoted]
+        doc_quoted = self._process_doc(fn, data)
 
         if data.keepalive is not None:
             x_keepalives = data.keepalive
 
+        if not self.report_only:
+            if fn["template"]:
+                if data.template_impls is None and not data.cpp_code:
+                    raise ValueError(
+                        f"{fn['name']}: must specify template impls for function template"
+                    )
+            else:
+                if data.template_impls is not None:
+                    raise ValueError(
+                        f"{fn['name']}: cannot specify template_impls for non-template functions"
+                    )
+
+            if data.ignore_pure and not fn["pure_virtual"]:
+                raise ValueError(
+                    f"{fn['name']}: cannot specify ignore_pure for function that isn't pure"
+                )
+
         # if "hook" in data:
         #     eval(data["hook"])(fn, data)
 
@@ -415,7 +484,6 @@ class Hooks:
                 x_callend=x_callend,
                 x_wrap_return=x_wrap_return,
                 # docstrings
-                x_doc=doc,
                 x_doc_quoted=doc_quoted,
             )
         )
@@ -453,6 +521,9 @@ class Hooks:
         if class_data.ignore:
             return
 
+        for typename in class_data.force_type_casters:
+            self._add_type_caster(typename)
+
         self._add_subpackage(cls, class_data)
 
         # fix enum paths
@@ -466,20 +537,37 @@ class Hooks:
             self._enum_hook(e, enum_data)
 
         # update inheritance
+
+        pybase_params = set()
+
         for base in cls["inherits"]:
-            if "::" not in base["class"]:
-                base["x_qualname"] = f'{cls["namespace"]}::{base["class"]}'
+            bqual = class_data.base_qualnames.get(base["class"])
+            if bqual:
+                base["x_qualname"] = bqual
+            elif "::" not in base["decl_name"]:
+                base["x_qualname"] = f'{cls["namespace"]}::{base["decl_name"]}'
             else:
-                base["x_qualname"] = base["class"]
+                base["x_qualname"] = base["decl_name"]
 
             base["x_qualname_"] = base["x_qualname"].translate(self._qualname_trans)
 
+            base_decl_params = base.get("decl_params")
+            if base_decl_params:
+                for decl_param in base_decl_params:
+                    pybase_params.add(decl_param["param"])
+                base["x_params"] = ", " + ", ".join(
+                    decl_param["param"] for decl_param in base_decl_params
+                )
+            else:
+                base["x_params"] = ""
+
         ignored_bases = {ib: True for ib in class_data.ignored_bases}
 
         cls["x_inherits"] = [
             base
             for base in cls["inherits"]
             if not ignored_bases.pop(base["class"], None)
+            and base["access"] != "private"
         ]
         if ignored_bases:
             bases = ", ".join(base["class"] for base in cls["inherits"])
@@ -489,13 +577,74 @@ class Hooks:
                 + f"{invalid_bases}; valid bases are {bases}"
             )
 
-        cls_qualname = cls["namespace"] + "::" + cls_name
-        cls["x_qualname"] = cls_qualname
-        cls["x_qualname_"] = cls_qualname.translate(self._qualname_trans)
+        # No template stuff
+        simple_cls_qualname = f'{cls["namespace"]}::{cls_name}'
 
-        self.class_hierarchy[cls_qualname] = [
+        # Template stuff
+        if cls["parent"]:
+            cls_qualname = f'{cls["parent"]["x_qualname"]}::{cls_name}'
+        else:
+            cls_qualname = simple_cls_qualname
+
+        cls["x_qualname_"] = cls_qualname.translate(self._qualname_trans)
+        self.class_hierarchy[simple_cls_qualname] = [
             base["x_qualname"] for base in cls["x_inherits"]
-        ]
+        ] + class_data.force_depends
+
+        # <N, .. >
+        template_argument_list = ""
+        # <typename N, .. >
+        template_parameter_list = ""
+
+        if class_data.template_params:
+            template_args = []
+            template_params = []
+
+            base_template_args = []
+            base_template_params = []
+
+            for param in class_data.template_params:
+                if " " in param:
+                    arg = param.split(" ", 1)[1]
+                else:
+                    arg = param
+                    param = f"typename {param}"
+
+                template_args.append(arg)
+                template_params.append(param)
+
+                if arg in pybase_params:
+                    base_template_args.append(arg)
+                    base_template_params.append(param)
+
+            template_argument_list = ", ".join(template_args)
+            template_parameter_list = ", ".join(template_params)
+
+            cls_qualname = f"{cls_qualname}<{template_argument_list}>"
+        else:
+            base_template_params = None
+            base_template_args = None
+
+        if base_template_params:
+            cls["x_pybase_args"] = f", {', '.join(base_template_args)}"
+            cls["x_pybase_params"] = f", {', '.join(base_template_params)}"
+        else:
+            cls["x_pybase_args"] = ""
+            cls["x_pybase_params"] = ""
+
+        if not self.report_only:
+            if "template" in cls:
+                if template_parameter_list == "":
+                    raise ValueError(
+                        f"{cls_name}: must specify template_params for templated class, or ignore it"
+                    )
+            else:
+                if template_parameter_list != "":
+                    raise ValueError(
+                        f"{cls_name}: cannot specify template_params for non-template class"
+                    )
+
+        cls["x_qualname"] = cls_qualname
 
         has_constructor = False
         is_polymorphic = class_data.is_polymorphic
@@ -512,34 +661,46 @@ class Hooks:
                 if fn["override"] or fn["virtual"]:
                     is_polymorphic = True
 
-                # Ignore operators, move constructors, copy constructors
+                operator = fn.get("operator")
+
+                # Ignore some operators, move constructors, copy constructors
                 if (
-                    fn.get("operator")
+                    (operator and operator not in _operators)
                     or fn.get("destructor")
                     or (
                         fn.get("constructor")
                         and fn["parameters"]
                         and fn["parameters"][0]["class"] is cls
                     )
+                    or fn["deleted"]
                 ):
                     fn["data"] = FunctionData(ignore=True)
                     continue
 
-                if access != "private":
-                    internal = access != "public"
+                is_private = access == "private"
+
+                # this has to be done even on private functions, because
+                # we do overload detection here
+                signature = self._get_function_signature(fn)
+                method_data = self.gendata.get_function_data(
+                    fn, signature, cls_key, class_data, is_private
+                )
+
+                if not is_private:
 
-                    signature = self._get_function_signature(fn)
-                    method_data = self.gendata.get_function_data(
-                        fn, signature, cls_key, class_data
-                    )
                     if method_data.ignore:
                         fn["data"] = method_data
                         continue
 
+                    if operator:
+                        self.has_operators = True
+                        if method_data.no_release_gil is None:
+                            method_data.no_release_gil = True
+
+                    internal = access != "public"
+
                     try:
-                        self._function_hook(
-                            fn, method_data, internal=internal,
-                        )
+                        self._function_hook(fn, method_data, internal=internal)
                     except Exception as e:
                         raise HookError(f"{cls_key}::{fn['name']}") from e
 
@@ -567,19 +728,32 @@ class Hooks:
                     v["x_name"] = v["name"] if access == "public" else "_" + v["name"]
 
                 if propdata.access == PropAccess.AUTOMATIC:
-                    # Properties that aren't fundamental or a reference are readonly unless
-                    # overridden by the hook configuration
-                    x_readonly = not v["fundamental"] and not v["reference"]
+                    # We assume that a struct intentionally has readwrite data
+                    # attributes regardless of type
+                    if cls["declaration_method"] != "class":
+                        x_readonly = False
+                    else:
+                        # Properties that aren't fundamental or a reference are readonly unless
+                        # overridden by the hook configuration
+                        x_readonly = not v["fundamental"] and not v["reference"]
                 elif propdata.access == PropAccess.READONLY:
                     x_readonly = True
                 else:
                     x_readonly = False
 
                 v["x_readonly"] = x_readonly
+                v["x_doc_quoted"] = self._process_doc(v, propdata)
 
         cls["x_has_trampoline"] = has_trampoline
+        cls["x_template_parameter_list"] = template_parameter_list
         if cls["x_has_trampoline"]:
-            cls["x_trampoline_name"] = f"rpygen::Py{cls['x_qualname_']}<{cls_name}>"
+            tmpl = ""
+            if template_argument_list:
+                tmpl = f", {template_argument_list}"
+            cls[
+                "x_trampoline_name"
+            ] = f"rpygen::Py{cls['x_qualname_']}<typename {cls_qualname}{tmpl}>"
         cls["x_has_constructor"] = has_constructor
         cls["x_varname"] = "cls_" + cls_name
         cls["x_name"] = self._set_name(cls_name, class_data)
+        cls["x_doc_quoted"] = self._process_doc(cls, class_data)
