@@ -17,6 +17,9 @@ class Model(BaseModel):
 class ParamData(Model):
     """Various ways to modify parameters"""
 
+    # Rename parameter
+    name: Optional[str] = None
+
     # C++ type emitted
     x_type: Optional[str] = None
 
@@ -25,6 +28,8 @@ class ParamData(Model):
 
     force_out: bool = False
 
+    array_size: Optional[int] = None
+
     ignore: bool = False
 
 
@@ -73,6 +78,11 @@ class FunctionData(Model):
     # If True, don't wrap this
     ignore: bool = False
 
+    # If True, don't wrap this, but provide a pure virtual implementation
+    # -> in theory we could autodetect this, but sometimes ignore
+    #    is used for when CppHeaderParser totally blows it
+    ignore_pure: bool = False
+
     # Generate this in an `#ifdef`
     ifdef: Optional[str] = None
     # Generate this in an `#ifndef`
@@ -103,11 +113,18 @@ class FunctionData(Model):
     overloads: Dict[str, "FunctionData"] = {}
 
     #: Adds py::keep_alive<x,y> to the function. Overrides automatic
-    #: keepalive support, which retains references passed to constructors
+    #: keepalive support, which retains references passed to constructors.
+    #: https://pybind11.readthedocs.io/en/stable/advanced/functions.html#keep-alive
     keepalive: Optional[List[Tuple[int, int]]] = None
 
+    #: https://pybind11.readthedocs.io/en/stable/advanced/functions.html#return-value-policies
     return_value_policy: ReturnValuePolicy = ReturnValuePolicy.AUTOMATIC
 
+    #: If this is a function template, this is a list of instantiations
+    #: that you wish to provide. This is a list of lists, where the inner
+    #: list is the template parameters for that function
+    template_impls: Optional[List[List[str]]] = None
+
     @validator("overloads", pre=True)
     def validate_overloads(cls, value):
         for k, v in value.items():
@@ -121,7 +138,10 @@ FunctionData.update_forward_refs()
 
 class PropAccess(enum.Enum):
 
-    #: Determine read/read-write automatically
+    #: Determine read/read-write automatically:
+    #: - If a struct/union, default to readwrite
+    #: - If a class, default to readwrite if a basic type that isn't a
+    #:   reference, otherwise default to readonly
     AUTOMATIC = "auto"
 
     #: Allow python users access to the value, but ensure it can't
@@ -142,8 +162,23 @@ class PropData(Model):
     #: Python code access to this property
     access: PropAccess = PropAccess.AUTOMATIC
 
+    #: Docstring for the property (only available on class properties)
+    doc: Optional[str] = None
+
+
+class EnumValue(Model):
+    ignore: bool = False
+    rename: Optional[str] = None
+
+    #: Docstring for the enum value
+    doc: Optional[str] = None
+
 
 class EnumData(Model):
+
+    # Docstring for the enum
+    doc: Optional[str] = None
+
     ignore: bool = False
     rename: Optional[str] = None
     value_prefix: Optional[str] = None
@@ -152,18 +187,37 @@ class EnumData(Model):
     #: enums that are part of classes)
     subpackage: Optional[str] = None
 
+    values: Dict[str, EnumValue] = {}
+
 
 class ClassData(Model):
 
+    #: Docstring for the class
+    doc: Optional[str] = None
+
     ignore: bool = False
     ignored_bases: List[str] = []
 
+    # Specify fully qualified names for the bases
+    base_qualnames: Dict[str, str] = {}
+
     attributes: Dict[str, PropData] = {}
     enums: Dict[str, EnumData] = {}
     methods: Dict[str, FunctionData] = {}
 
     is_polymorphic: bool = False
     force_no_trampoline: bool = False
+    force_no_default_constructor: bool = False
+
+    #: If there are circular dependencies, this will help you resolve them
+    #: manually. TODO: make it so we don't need this
+    force_depends: List[str] = []
+
+    #: Use this to bring in type casters for a particular type that may have
+    #: been hidden (for example, with a typedef or definition in another file),
+    #: instead of explicitly including the header. This should be the full
+    #: namespace of the type.
+    force_type_casters: List[str] = []
 
     #: If the object shouldn't be deleted by pybind11, use this. Disables
     #: implicit constructors.
@@ -192,6 +246,17 @@ class ClassData(Model):
     #: Extra constexpr to insert into the trampoline and wrapping scopes
     constants: List[str] = []
 
+    #: If this is a template class, a list of the parameters if it can't
+    #: be autodetected (currently can't autodetect). If there is no space
+    #: in the parameter, then it is assumed to be a 'typename', otherwise
+    #: the parameter is split by space and the first item is the type and
+    #: the second parameter is the name (useful for integral templates)
+    template_params: Optional[List[str]] = None
+
+    #: If this is a template class, the specified C++ code is inserted
+    #: into the template definition
+    template_inline_code: str = ""
+
     @validator("attributes", pre=True)
     def validate_attributes(cls, value):
         for k, v in value.items():
@@ -214,6 +279,17 @@ class ClassData(Model):
         return value
 
 
+class TemplateData(Model):
+
+    # Fully qualified name of template
+    qualname: str
+
+    # Template parameters to use
+    params: List[str]
+
+    # TODO: other parameters useful for concrete types
+
+
 class HooksDataYaml(Model):
     """
         Format of the file in [tool.robotpy-build.wrappers."PACKAGENAME"]
@@ -222,6 +298,8 @@ class HooksDataYaml(Model):
 
     strip_prefixes: List[str] = []
     extra_includes: List[str] = []
+    # sticks these includes first, for really broken headers
+    extra_includes_first: List[str] = []
     inline_code: Optional[str] = None
 
     attributes: Dict[str, PropData] = {}
@@ -229,6 +307,9 @@ class HooksDataYaml(Model):
     functions: Dict[str, FunctionData] = {}
     enums: Dict[str, EnumData] = {}
 
+    #: Instantiates a template. Key is the name to give to the Python type
+    templates: Dict[str, TemplateData] = {}
+
     @validator("attributes", pre=True)
     def validate_attributes(cls, value):
         for k, v in value.items():
