@@ -1,6 +1,7 @@
 from distutils.util import get_platform as _get_platform
 from dataclasses import dataclass, field
 from typing import List, Tuple, Union
+import re
 
 
 # wpilib platforms at https://github.com/wpilibsuite/native-utils/blob/master/src/main/java/edu/wpi/first/nativeutils/WPINativeUtilsExtension.java
@@ -16,6 +17,9 @@ class WPILibMavenPlatform:
     #: compile time linkage
     linkext: str = None
 
+    #: static linkage
+    staticext: str = ".a"
+
     defines: List[str] = field(default_factory=list)
 
     def __post_init__(self):
@@ -34,10 +38,9 @@ _platforms = {
     "linux-armv7l": WPILibMavenPlatform("athena", defines=["__FRC_ROBORIO__"]),
     "linux-x86_64": WPILibMavenPlatform(X86_64),
     # TODO: linuxraspbian
-    "win32": WPILibMavenPlatform("x86", "windows", "", ".dll", ".lib"),
-    "win-amd64": WPILibMavenPlatform(X86_64, "windows", "", ".dll", ".lib"),
-    # TODO: need to filter this value
-    "macosx-10.9-x86_64": WPILibMavenPlatform(X86_64, "osx", libext=".dylib"),
+    "win32": WPILibMavenPlatform("x86", "windows", "", ".dll", ".lib", ".lib"),
+    "win-amd64": WPILibMavenPlatform(X86_64, "windows", "", ".dll", ".lib", ".lib"),
+    "osx": WPILibMavenPlatform(X86_64, "osx", libext=".dylib"),
 }
 
 
@@ -51,7 +54,21 @@ def get_platform() -> WPILibMavenPlatform:
     #       be useful to note for the future.
 
     pyplatform = _get_platform()
+
+    # Check for 64 bit x86 macOS (version agnostic)
+    if re.fullmatch(r"macosx-.*-x86_64", pyplatform):
+        pyplatform = "osx"
+
     try:
         return _platforms[pyplatform]
     except KeyError:
         raise KeyError(f"platform {pyplatform} is not supported by robotpy-build!")
+
+
+def get_platform_override_keys(platform: WPILibMavenPlatform):
+    # Used in places where overrides exist
+    return [
+        f"arch_{platform.arch}",
+        f"os_{platform.os}",
+        f"platform_{platform.os}_{platform.arch}",
+    ]
