@@ -5,6 +5,13 @@ from typing import Dict, List, Optional
 from pydantic import BaseModel
 
 
+class PatchInfo(BaseModel):
+    #: Name of patch file to apply
+    patch: str
+    #: Number of directories to strip
+    strip: int = 0
+
+
 class MavenLibDownload(BaseModel):
     """
         The information required here can be found in the vendor JSON file
@@ -47,6 +54,26 @@ class MavenLibDownload(BaseModel):
     #: If use_sources is set, this is the list of sources to compile
     sources: Optional[List[str]] = None
 
+    #: Configure the sources classifier
+    sources_classifier: str = "sources"
+
+    #: If use_sources is set, apply the following patches to the sources. Patches
+    #: must be in unified diff format. Key is filename, value is number of paths
+    #: to strip from the file
+    patches: Optional[List[PatchInfo]] = None
+
+
+class StaticLibConfig(BaseModel):
+    class Config:
+        extra = "forbid"
+
+    #: If this project depends on external libraries stored in a maven repo
+    #: specify it here
+    maven_lib_download: MavenLibDownload
+
+    #: If True, skip this library; typically used in conjection with an override
+    ignore: bool = False
+
 
 class WrapperConfig(BaseModel):
     """
@@ -108,6 +135,9 @@ class WrapperConfig(BaseModel):
     # Preprocessor definitions
     pp_defines: List[str] = []
 
+    #: If True, skip this wrapper; typically used in conjection with an override
+    ignore: bool = False
+
 
 class DistutilsMetadata(BaseModel):
     class Config:
@@ -156,3 +186,6 @@ class RobotpyBuildConfig(BaseModel):
 
     # [tool.robotpy-build.wrappers."XXX"]
     wrappers: Dict[str, WrapperConfig] = {}
+
+    # [tool.robotpy-build.static_libs."XXX"]
+    static_libs: Dict[str, StaticLibConfig] = {}
