@@ -22,10 +22,13 @@ from .command.build_py import BuildPy
 from .command.build_dl import BuildDl
 from .command.build_gen import BuildGen
 from .command.build_ext import BuildExt
+from .command.develop import Develop
 
+from .overrides import apply_overrides
 from .pyproject_configs import RobotpyBuildConfig
 from .pkgcfg_provider import PkgCfgProvider
-from .platforms import get_platform
+from .platforms import get_platform, get_platform_override_keys
+from .static_libs import StaticLib
 from .wrapper import Wrapper
 
 
@@ -38,6 +41,9 @@ class Setup:
     def __init__(self):
         self.root = abspath(os.getcwd())
         self.wrappers = []
+        self.static_libs = []
+
+        self.platform = get_platform()
 
         project_fname = join(self.root, "pyproject.toml")
 
@@ -48,6 +54,13 @@ class Setup:
             raise ValueError("current directory is not a robotpy-build project") from e
 
         self.project_dict = self.pyproject.get("tool", {}).get("robotpy-build", {})
+
+        # Overrides are applied before pydantic does processing, so that
+        # we can easily override anything without needing to make the
+        # pydantic schemas messy with needless details
+        override_keys = get_platform_override_keys(self.platform)
+        apply_overrides(self.project_dict, override_keys)
+
         try:
             self.project = RobotpyBuildConfig(**self.project_dict)
         except Exception as e:
@@ -55,8 +68,6 @@ class Setup:
                 f"robotpy-build configuration in pyproject.toml is incorrect"
             ) from e
 
-        self.platform = get_platform()
-
     @property
     def base_package(self):
         return self.project.base_package
@@ -69,6 +80,10 @@ class Setup:
     def git_dir(self):
         return join(self.root, ".git")
 
+    @property
+    def pypi_package(self) -> str:
+        return self.setup_kwargs["name"]
+
     def prepare(self):
 
         self.setup_kwargs = self.project_dict.get("metadata", {})
@@ -76,10 +91,6 @@ class Setup:
         self.setup_kwargs["include_package_data"] = True
         self.setup_kwargs["python_requires"] = ">=3.6"
 
-        # TODO: autogen packages don't exist at sdist time
-        #       ... but we want them to be added to the wheel
-        self.setup_kwargs["packages"] = find_packages()
-
         self._generate_long_description()
 
         # get_version expects the directory to exist
@@ -91,6 +102,7 @@ class Setup:
 
         self.pkgcfg = PkgCfgProvider()
 
+        self._collect_static_libs()
         self._collect_wrappers()
 
         self.setup_kwargs["cmdclass"] = {
@@ -98,11 +110,20 @@ class Setup:
             "build_dl": BuildDl,
             "build_gen": BuildGen,
             "build_ext": BuildExt,
+            "develop": Develop,
         }
         if bdist_wheel:
             self.setup_kwargs["cmdclass"]["bdist_wheel"] = bdist_wheel
         for cls in self.setup_kwargs["cmdclass"].values():
             cls.wrappers = self.wrappers
+            cls.static_libs = self.static_libs
+            cls.rpybuild_pkgcfg = self.pkgcfg
+
+        # We already know some of our packages, so collect those in addition
+        # to using find_packages()
+        packages = {w.package_name for w in self.wrappers}
+        packages.update(find_packages())
+        self.setup_kwargs["packages"] = list(packages)
 
     def _generate_long_description(self):
         readme_rst = join(self.root, "README.rst")
@@ -121,6 +142,8 @@ class Setup:
         ext_modules = []
 
         for package_name, cfg in self.project.wrappers.items():
+            if cfg.ignore:
+                continue
             w = Wrapper(package_name, cfg, self)
             self.wrappers.append(w)
             self.pkgcfg.add_pkg(w)
@@ -131,6 +154,14 @@ class Setup:
         if ext_modules:
             self.setup_kwargs["ext_modules"] = ext_modules
 
+    def _collect_static_libs(self):
+        for name, cfg in self.project.static_libs.items():
+            if cfg.ignore:
+                continue
+            s = StaticLib(name, cfg, self)
+            self.static_libs.append(s)
+            self.pkgcfg.add_pkg(s)
+
     def run(self):
         # assemble all the pieces and make it work
         _setup(**self.setup_kwargs)
