@@ -2,16 +2,26 @@ import argparse
 import glob
 import inspect
 from os.path import basename, dirname, exists, join, relpath, splitext
+from pathlib import PurePosixPath
+import pprint
 import subprocess
 import sys
 
 from .setup import Setup
 from .generator_data import MissingReporter
+from .command.util import get_build_temp_path
+
+from . import platforms
 
 
 def get_setup() -> Setup:
     s = Setup()
     s.prepare()
+
+    temp_path = join(get_build_temp_path(), "dlstatic")
+    for static_lib in s.static_libs:
+        static_lib.set_root(temp_path)
+
     return s
 
 
@@ -19,9 +29,10 @@ class BuildDep:
     @classmethod
     def add_subparser(cls, parent_parser, subparsers):
         parser = subparsers.add_parser(
-            "build-dep", help="Install build dependencies", parents=[parent_parser],
+            "build-dep", help="Install build dependencies", parents=[parent_parser]
         )
         parser.add_argument("--install", help="Actually do it", action="store_true")
+        parser.add_argument("--find-links", help="Find links arg", default=None)
         return parser
 
     def run(self, args):
@@ -37,6 +48,11 @@ class BuildDep:
             "--disable-pip-version-check",
             "install",
         ]
+        if args.find_links:
+            pipargs.extend(
+                ["--find-links", args.find_links,]
+            )
+
         pipargs.extend(requirements)
         print(" ".join(pipargs))
 
@@ -119,14 +135,15 @@ class HeaderScanner:
                         thisdir = dirname(f)
                         if lastdir is None:
                             if thisdir:
-                                print("    #", thisdir)
+                                print("    #", PurePosixPath(thisdir))
                         elif lastdir != thisdir:
                             print()
                             if thisdir:
-                                print("    #", thisdir)
+                                print("    #", PurePosixPath(thisdir))
                         lastdir = thisdir
 
                         base = splitext(basename(f))[0]
+                        f = PurePosixPath(f)
                         print(f'    {{ {base} = "{f}" }},')
                 print("]")
 
@@ -176,6 +193,69 @@ class ImportCreator:
         )
 
 
+class LibraryRelinker:
+    @classmethod
+    def add_subparser(cls, parent_parser, subparsers):
+        parser = subparsers.add_parser(
+            "relink-libraries",
+            help="Relink libraries to new paths",
+            parents=[parent_parser],
+        )
+        parser.add_argument("libraries", help="Folder with libraries - Ex: ./libs")
+        parser.add_argument(
+            "dependents",
+            help="Folder with dependents - Ex: ../../venv/libs/site-packages",
+        )
+        return parser
+
+    def run(self, args):
+        raise NotImplementedError(
+            "tool needs to be fixed -- just reinstall the offending package instead"
+        )
+        # Essentially a macos check
+        try:
+            import delocate
+        except:
+            print("relink-libraries is only designed to work on macOS.")
+            print("If you are on macOS, then:")
+            print("Error, The following module is required to run this tool: delocate")
+            exit(1)
+
+        from . import relink_libs
+        from os.path import abspath
+
+        dependencies = relink_libs.get_build_dependencies(args.dependents)
+        libs = relink_libs.find_all_libs(args.libraries)
+        for key in libs:
+            libs[key] = "@" + abspath(libs[key]) + "@"
+
+        print("Format | dependent : old path to lib -> new path to lib")
+        # We are only using abs paths so build_path doesn't matter
+        relink_libs.redirect_links(
+            ".", libs, dependencies, auto_detect=False, supress_errors=True
+        )
+
+
+class PlatformInfo:
+    @classmethod
+    def add_subparser(cls, parent_parser, subparsers):
+        parser = subparsers.add_parser(
+            "platform-info",
+            help="Displays platform-specific information",
+            parents=[parent_parser],
+        )
+        return parser
+
+    def run(self, args):
+        p = platforms.get_platform()
+        print("platform:")
+        pprint.pprint(p)
+        print()
+
+        print("override keys:")
+        pprint.pprint(platforms.get_platform_override_keys(p))
+
+
 def main():
 
     parser = argparse.ArgumentParser(prog="robotpy-build")
@@ -183,7 +263,14 @@ def main():
     subparsers = parser.add_subparsers(dest="cmd")
     subparsers.required = True
 
-    for cls in (BuildDep, GenCreator, HeaderScanner, ImportCreator):
+    for cls in (
+        BuildDep,
+        GenCreator,
+        HeaderScanner,
+        ImportCreator,
+        LibraryRelinker,
+        PlatformInfo,
+    ):
         cls.add_subparser(parent_parser, subparsers).set_defaults(cls=cls)
 
     args = parser.parse_args()
