@@ -15,10 +15,12 @@ from os.path import (
     splitext,
 )
 import posixpath
+import subprocess
 import sys
 import shutil
+import tempfile
 import toposort
-from typing import Optional, List
+from typing import Dict, List, Optional, Set
 import yaml
 
 from header2whatever.config import Config
@@ -31,7 +33,7 @@ from .pyproject_configs import WrapperConfig, MavenLibDownload
 from .generator_data import MissingReporter
 from .hooks import Hooks
 from .hooks_datacfg import HooksDataYaml
-from .download import download_and_extract_zip
+from .download import download_maven
 
 
 class Wrapper:
@@ -49,10 +51,12 @@ class Wrapper:
         self.cfg = cfg
 
         self.setup_root = setup.root
+        self.pypi_package = setup.pypi_package
         self.root = join(setup.root, *package_name.split("."))
 
         # must match PkgCfg.name
         self.name = cfg.name
+        self.static_lib = False
 
         # Compute the extension name, even if we don't create one
         extname = cfg.extension
@@ -65,7 +69,7 @@ class Wrapper:
             if libinit_py == "__init__.py":
                 self.libinit_import = package_name
             else:
-                pkg = splitext(self.cfg.libinit)[0]
+                pkg = splitext(libinit_py)[0]
                 self.libinit_import = f"{package_name}.{pkg}"
         else:
             libinit_py = f"_init{extname}.py"
@@ -80,10 +84,12 @@ class Wrapper:
         self.depends = self.cfg.depends
 
         # Files that are generated AND need to be in the final wheel. Used by build_py
-        self.generated_files = []
+        self.generated_files: List[str] = []
 
         self._all_deps = None
 
+        self._gen_includes = []
+
         self.extension = None
         if self.cfg.sources or self.cfg.generate:
             define_macros = [("RPYBUILD_MODULE_NAME", extname)] + [
@@ -102,6 +108,9 @@ class Wrapper:
                 language="c++",
             )
 
+            # Add self to extension so that build_ext can query it on OSX
+            self.extension.rpybuild_wrapper = self
+
         if self.cfg.generate and not self.cfg.generation_data:
             raise ValueError(
                 "generation_data must be specified when generate is specified"
@@ -119,18 +128,8 @@ class Wrapper:
 
         self.dev_config = get_dev_config(self.name)
 
-    def _dl_url(self, classifier):
-        # TODO: support development against locally installed things?
-        dl = self.cfg.maven_lib_download
-        repo_url = dl.repo_url
-        grp = dl.group_id.replace(".", "/")
-        art = dl.artifact_id
-        ver = dl.version
-
-        return f"{repo_url}/{grp}/{art}/{ver}/{art}-{ver}-{classifier}.zip"
-
-    def _extract_zip_to(self, thing, dst, cache):
-        download_and_extract_zip(self._dl_url(thing), to=dst, cache=cache)
+    def _extract_zip_to(self, classifier, dst, cache):
+        download_maven(self.cfg.maven_lib_download, classifier, dst, cache)
 
     def _add_generated_file(self, fullpath):
         if not isdir(fullpath):
@@ -148,6 +147,11 @@ class Wrapper:
             return [join(self.root, "lib")]
         return []
 
+    def get_library_dirs_rel(self) -> Optional[List[str]]:
+        if self.get_library_names():
+            return ["lib"]
+        return []
+
     def get_library_names(self) -> Optional[List[str]]:
         mcfg = self.cfg.maven_lib_download
         if mcfg:
@@ -160,6 +164,28 @@ class Wrapper:
         else:
             return []
 
+    def get_library_full_names(self) -> Optional[List[str]]:
+        dlcfg = self.cfg.maven_lib_download
+        if not dlcfg:
+            return []
+
+        libnames = self.get_library_names()
+        dlopen_libnames = self.get_dlopen_library_names()
+
+        libnames = [lib for lib in libnames if lib not in dlopen_libnames]
+        libnames_full = []
+
+        if libnames or dlopen_libnames:
+            libext = dlcfg.libexts.get(self.platform.libext, self.platform.libext)
+
+            libnames_full = [
+                f"{self.platform.libprefix}{lib}{libext}" for lib in libnames
+            ]
+            libnames_full += [
+                f"{self.platform.libprefix}{lib}{libext}" for lib in dlopen_libnames
+            ]
+        return libnames_full
+
     def get_dlopen_library_names(self) -> Optional[List[str]]:
         mcfg = self.cfg.maven_lib_download
         if mcfg and mcfg.dlopenlibs:
@@ -167,7 +193,10 @@ class Wrapper:
         else:
             return []
 
-    def get_type_casters(self, casters):
+    def get_extra_objects(self) -> Optional[List[str]]:
+        pass
+
+    def get_type_casters(self, casters: Dict):
         for header, types in self.cfg.type_casters.items():
             for typ in types:
                 casters[typ] = header
@@ -188,7 +217,9 @@ class Wrapper:
     def _all_library_dirs(self):
         libs = self.get_library_dirs()
         for dep in self.cfg.depends:
-            libs.extend(self.pkgcfg.get_pkg(dep).get_library_dirs())
+            libdirs = self.pkgcfg.get_pkg(dep).get_library_dirs()
+            if libdirs:
+                libs.extend(libdirs)
         return libs
 
     def _all_library_names(self):
@@ -196,7 +227,19 @@ class Wrapper:
             set(self.get_library_names()) | set(self.get_dlopen_library_names())
         )
         for dep in self.cfg.depends:
-            libs.extend(self.pkgcfg.get_pkg(dep).get_library_names())
+            pkg = self.pkgcfg.get_pkg(dep)
+            libnames = pkg.get_library_names()
+            if libnames:
+                libs.extend(libnames)
+        return list(reversed(libs))
+
+    def _all_extra_objects(self):
+        libs = []
+        for dep in self.cfg.depends:
+            pkg = self.pkgcfg.get_pkg(dep)
+            libnames = pkg.get_extra_objects()
+            if libnames:
+                libs.extend(libnames)
         return list(reversed(libs))
 
     def _all_casters(self):
@@ -235,7 +278,7 @@ class Wrapper:
             libnames_full = self._clean_and_download(dlcfg, cache, srcdir)
 
         self._write_libinit_py(libnames_full)
-        self._write_pkgcfg_py(pkgcfgpy)
+        self._write_pkgcfg_py(pkgcfgpy, libnames_full)
 
     def _clean_and_download(
         self, dlcfg: MavenLibDownload, cache: str, srcdir: str
@@ -253,39 +296,46 @@ class Wrapper:
         self._extract_zip_to("headers", incdir, cache)
 
         if dlcfg.use_sources:
-            self._extract_zip_to("source", srcdir, cache)
+            self._extract_zip_to(dlcfg.sources_classifier, srcdir, cache)
             if dlcfg.sources:
                 sources = [join(srcdir, normpath(s)) for s in dlcfg.sources]
                 self.extension.sources.extend(sources)
+            if dlcfg.patches:
+                import patch
+
+                for p in dlcfg.patches:
+                    patch_path = join(self.setup_root, normpath(p.patch))
+                    ps = patch.PatchSet()
+                    with open(patch_path, "rb") as fp:
+                        if not ps.parse(fp):
+                            raise ValueError(f"Error parsing patch '{patch_path}'")
+
+                    if not ps.apply(strip=p.strip, root=srcdir):
+                        raise ValueError(f"Error applying patch '{patch_path}")
             return []
         elif dlcfg.sources is not None:
             raise ValueError("sources must be None if use_sources is False!")
+        elif dlcfg.patches is not None:
+            raise ValueError("patches must be None if use_sources is False!")
 
         libnames = self.get_library_names()
         dlopen_libnames = self.get_dlopen_library_names()
 
         libnames = [lib for lib in libnames if lib not in dlopen_libnames]
-        libnames_full = []
+        libnames_full = self.get_library_full_names()
 
-        if libnames or dlopen_libnames:
+        if libnames_full:
             libext = dlcfg.libexts.get(self.platform.libext, self.platform.libext)
             linkext = dlcfg.linkexts.get(self.platform.linkext, self.platform.linkext)
 
-            libnames_full = [
-                f"{self.platform.libprefix}{lib}{libext}" for lib in libnames
-            ]
-            libnames_full += [
-                f"{self.platform.libprefix}{lib}{libext}" for lib in dlopen_libnames
-            ]
+            os.makedirs(libdir)
 
             extract_names = libnames_full[:]
-
             if libext != linkext:
                 extract_names += [
                     f"{self.platform.libprefix}{lib}{linkext}" for lib in libnames
                 ]
 
-            os.makedirs(libdir)
             to = {
                 posixpath.join(
                     self.platform.os, self.platform.arch, "shared", libname
@@ -314,7 +364,7 @@ class Wrapper:
         # This file is automatically generated, DO NOT EDIT
         # fmt: off
 
-        from os.path import abspath, join, dirname
+        from os.path import abspath, join, dirname, exists
         _root = abspath(dirname(__file__))
 
         ##IMPORTS##
@@ -326,9 +376,19 @@ class Wrapper:
 
         if libnames:
             init += "from ctypes import cdll\n\n"
-            for libname in libnames:
-                init += f'_lib = cdll.LoadLibrary(join(_root, "lib", "{libname}"))\n'
 
+            for libname in libnames:
+                init += "try:\n"
+                init += (
+                    f'    _lib = cdll.LoadLibrary(join(_root, "lib", "{libname}"))\n'
+                )
+                init += "except FileNotFoundError:\n"
+                init += f'    if not exists(join(_root, "lib", "{libname}")):\n'
+                init += f'        raise FileNotFoundError("{libname} was not found on your system. Is this package correctly installed?")\n'
+                if self.platform.os == "windows":
+                    init += f'    raise Exception("{libname} could not be loaded. Do you have Visual Studio C++ Redistributible 2019 installed?")\n\n'
+                else:
+                    init += f'    raise FileNotFoundError("{libname} could not be loaded. There is a missing dependency.")\n\n'
         imports = []
         for dep in self.cfg.depends:
             pkg = self.pkgcfg.get_pkg(dep)
@@ -347,12 +407,20 @@ class Wrapper:
 
         self._add_generated_file(self.libinit_import_py)
 
-    def _write_pkgcfg_py(self, fname):
+    def _write_pkgcfg_py(self, fname, libnames_full):
 
         library_dirs = "[]"
+        library_dirs_rel = []
         library_names = self.get_library_names()
         if library_names:
             library_dirs = '[join(_root, "lib")]'
+            library_dirs_rel = ["lib"]
+
+        deps = []
+        for dep in self.cfg.depends:
+            pkg = self.pkgcfg.get_pkg(dep)
+            if not pkg.static_lib:
+                deps.append(dep)
 
         # write pkgcfg.py
         pkgcfg = inspect.cleandoc(
@@ -364,16 +432,23 @@ class Wrapper:
         _root = abspath(dirname(__file__))
 
         libinit_import = "{self.libinit_import}"
-        depends = {repr(self.cfg.depends)}
+        depends = {repr(deps)}
+        pypi_package = {repr(self.pypi_package)}
 
         def get_include_dirs():
             return [join(_root, "include"), join(_root, "rpy-include")##EXTRAINCLUDES##]
 
         def get_library_dirs():
             return {library_dirs}
+
+        def get_library_dirs_rel():
+            return {repr(library_dirs_rel)}
         
         def get_library_names():
             return {repr(library_names)}
+
+        def get_library_full_names():
+            return {repr(libnames_full)}
         """
         )
 
@@ -430,9 +505,9 @@ class Wrapper:
 
         hppoutdir = join(self.rpy_incdir, "rpygen")
         tmpl_dir = join(thisdir, "templates")
-        cpp_tmpl = join(tmpl_dir, "gen_pybind11.cpp.j2")
-        hpp_tmpl = join(tmpl_dir, "gen_cls_trampoline.hpp.j2")
-        classdeps_tmpl = join(tmpl_dir, "gen_classdeps.json.j2")
+        cpp_tmpl = join(tmpl_dir, "cls.cpp.j2")
+        hpp_tmpl = join(tmpl_dir, "cls_rpy_include.hpp.j2")
+        classdeps_tmpl = join(tmpl_dir, "clsdeps.json.j2")
 
         pp_includes = self._all_includes(False)
 
@@ -457,7 +532,6 @@ class Wrapper:
         else:
             data = HooksDataYaml()
 
-        sources = self.cfg.sources[:]
         pp_defines = [self._cpp_version] + self.platform.defines + self.cfg.pp_defines
         casters = self._all_casters()
 
@@ -471,7 +545,7 @@ class Wrapper:
         else:
             only_generate = None
 
-        generation_search_path = [self.root,] + self._all_includes(False)
+        generation_search_path = [self.root] + self._all_includes(False)
 
         for gen in self.cfg.generate:
             for name, header in gen.items():
@@ -482,7 +556,9 @@ class Wrapper:
                     if exists(header_path):
                         break
                 else:
-                    print(generation_search_path)
+                    import pprint
+
+                    pprint.pprint(generation_search_path)
                     raise ValueError("could not find " + header)
 
                 if report_only:
@@ -490,7 +566,7 @@ class Wrapper:
                     class_templates = []
                 else:
                     cpp_dst = join(cxx_gen_dir, f"{name}.cpp")
-                    sources.append(cpp_dst)
+                    self.extension.sources.append(cpp_dst)
                     classdeps_dst = join(cxx_gen_dir, f"{name}.json")
                     classdeps[name] = classdeps_dst
 
@@ -533,8 +609,11 @@ class Wrapper:
                 cfg.validate()
                 cfg.root = self.incdir
 
-                hooks = Hooks(data, casters)
-                processor.process_config(cfg, data, hooks)
+                hooks = Hooks(data, casters, report_only)
+                try:
+                    processor.process_config(cfg, data, hooks)
+                except Exception as e:
+                    raise ValueError(f"processing {header}") from e
 
                 hooks.report_missing(data_fname, missing_reporter)
 
@@ -555,21 +634,27 @@ class Wrapper:
         else:
             gen_includes = []
 
+        self._gen_includes = gen_includes
+
+        for f in glob.glob(join(glob.escape(hppoutdir), "*.hpp")):
+            self._add_generated_file(f)
+
+    def finalize_extension(self):
+        if self.extension is None:
+            return
+
         # Add the root to the includes (but only privately)
         root_includes = [self.root]
 
         # update the build extension so that build_ext works
-        self.extension.sources = sources
         # use normpath to get rid of .. otherwise gcc is weird
         self.extension.include_dirs = [
             normpath(p)
-            for p in (self._all_includes(True) + gen_includes + root_includes)
+            for p in (self._all_includes(True) + self._gen_includes + root_includes)
         ]
         self.extension.library_dirs = self._all_library_dirs()
         self.extension.libraries = self._all_library_names()
-
-        for f in glob.glob(join(glob.escape(hppoutdir), "*.hpp")):
-            self._add_generated_file(f)
+        self.extension.extra_objects = self._all_extra_objects()
 
     def _write_wrapper_hpp(self, outdir, classdeps):
 
@@ -604,7 +689,7 @@ class Wrapper:
                 types2name[clsname] = name
                 types2deps[clsname] = [_clean(base) for base in bases]
 
-        to_sort = {}
+        to_sort: Dict[str, Set[str]] = {}
         for clsname, bases in types2deps.items():
             clsname = types2name[clsname]
             deps = to_sort.setdefault(clsname, set())
